/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade;

import java.util.Date;
import java.util.List;

/**
 * This class provides a brief overview of Bonita Business Activity Monitoring.<br>
 * Note: Using this API can affect engine performances.
 * @author Matthieu Chaffotte
 *
 */
public interface BAMAPI {

  /**
   * Gets the list of the number of executing cases (process instances) per day of every users.
   * @param since the starting date
   * @return the list of executing cases number per day
   */
  List<Integer> getNumberOfExecutingCasesPerDay(Date since);

  /**
   * Gets the list of the number of the finished cases (process instances) per day of every users.
   * @param since the starting date
   * @return the list of finish cases number per day
   */
  List<Integer> getNumberOfFinishedCasesPerDay(Date since);

  /**
   * Gets the total number of finished steps of every users since the given date
   * according to the step priority.
   * @param priority the step priority
   * @param since 
   * @return the number of finished steps.
   */
  int getNumberOfFinishedSteps(int priority, Date since);

  /**
   * Gets the total number of open steps (i.e. tasks) of every users.
   * An open step is a step which is on time.
   * @return the number of open steps
   */
  int getNumberOfOpenSteps();

  /**
   * Gets the total number of open steps (i.e. tasks) of every users according to the step priority.
   * @param priority the step priority
   * @return the number of open steps
   */
  int getNumberOfOpenSteps(int priority);

  /**
   * Gets the number of open steps of every users for every days since the given day.
   * @param since the day 
   * @return a list the number of open steps for each day from the given day
   */
  List<Integer> getNumberOfOpenStepsPerDay(Date since);

  /**
   * Gets the total number of overdue steps (i.e. tasks) of every users.
   * An overdue task is a task which its ended date is exceeded.
   * @return the number of overdue steps.
   */
  int getNumberOfOverdueSteps();

  /**
   * Gets the total number of steps at risk (i.e. tasks) of every users. A step at risk is
   * a step which its ended date is close to exceed according to the remaining days.
   * @param remainingDays the remaining days before the ended step date is exceeded.
   * @return the number of steps at risk of every users.
   */
  int getNumberOfStepsAtRisk(int remainingDays);

  /**
   * Gets the total number of finished steps of the logged user since the given date according
   * to the step priority.
   * @param priority the step priority
   * @param since 
   * @return the number of finished steps.
   */
  int getNumberOfUserFinishedSteps(int priority, Date since);

  /**
   * Gets the number of open steps of the logged user.
   * @return the number of open steps of the logged user
   */
  int getNumberOfUserOpenSteps();

  /**
   * Gets the number of open steps of the logged user according to the step priority.
   * @param priority the step priority
   * @return the number of open step of the logged user
   */
  int getNumberOfUserOpenSteps(int priority);

  /**
   * Gets the number of overdue steps (i.e. tasks) of the logged user.
   * An overdue task is a task which its ended date is exceeded.
   * @return the number of overdue steps of the logged user.
   */
  int getNumberOfUserOverdueSteps();

  /**
   * Gets the number of steps at risk (i.e. tasks) of the logged user.
   * A step at risk is a step which its ended date is close to exceed according to the remaining days.
   * @param remainingDays the remaining days before the ended step date is exceeded.
   * @return the number of steps at risk of the logged user.
   */
  int getNumberOfUserStepsAtRisk(int remainingDays);

}
